@@ -20,9 +20,45 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gee/readonlycollection.h>
+#include <glib.h>
+#include <glib-object.h>
 
 
+#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
+#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
+#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
+#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))
+
+typedef struct _GeeIterable GeeIterable;
+typedef struct _GeeIterableIface GeeIterableIface;
+
+#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
+#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
+#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
+#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))
+
+typedef struct _GeeIterator GeeIterator;
+typedef struct _GeeIteratorIface GeeIteratorIface;
+
+#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
+#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
+#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
+#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))
+
+typedef struct _GeeCollection GeeCollection;
+typedef struct _GeeCollectionIface GeeCollectionIface;
+
+#define GEE_TYPE_READ_ONLY_COLLECTION (gee_read_only_collection_get_type ())
+#define GEE_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection))
+#define GEE_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
+#define GEE_IS_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_COLLECTION))
+#define GEE_IS_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_COLLECTION))
+#define GEE_READ_ONLY_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
+
+typedef struct _GeeReadOnlyCollection GeeReadOnlyCollection;
+typedef struct _GeeReadOnlyCollectionClass GeeReadOnlyCollectionClass;
+typedef struct _GeeReadOnlyCollectionPrivate GeeReadOnlyCollectionPrivate;
+
 #define GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR (gee_read_only_collection_iterator_get_type ())
 #define GEE_READ_ONLY_COLLECTION_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIterator))
 #define GEE_READ_ONLY_COLLECTION_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorClass))
@@ -34,6 +70,58 @@ typedef struct _GeeReadOnlyCollectionIterator GeeReadOnlyCollectionIterator;
 typedef struct _GeeReadOnlyCollectionIteratorClass GeeReadOnlyCollectionIteratorClass;
 typedef struct _GeeReadOnlyCollectionIteratorPrivate GeeReadOnlyCollectionIteratorPrivate;
 
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIteratorIface {
+	GTypeInterface parent_iface;
+	gboolean (*next) (GeeIterator* self);
+	gpointer (*get) (GeeIterator* self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIterableIface {
+	GTypeInterface parent_iface;
+	GType (*get_element_type) (GeeIterable* self);
+	GeeIterator* (*iterator) (GeeIterable* self);
+};
+
+/**
+ * Serves as the base interface for implementing collection classes. Defines
+ * size, iteration, and modification methods.
+ */
+struct _GeeCollectionIface {
+	GTypeInterface parent_iface;
+	gboolean (*contains) (GeeCollection* self, gconstpointer item);
+	gboolean (*add) (GeeCollection* self, gconstpointer item);
+	gboolean (*remove) (GeeCollection* self, gconstpointer item);
+	void (*clear) (GeeCollection* self);
+	gint (*get_size) (GeeCollection* self);
+};
+
+/**
+ * Represents a read-only collection of items.
+ */
+struct _GeeReadOnlyCollection {
+	GObject parent_instance;
+	GeeReadOnlyCollectionPrivate * priv;
+};
+
+struct _GeeReadOnlyCollectionClass {
+	GObjectClass parent_class;
+};
+
+struct _GeeReadOnlyCollectionPrivate {
+	GType g_type;
+	GBoxedCopyFunc g_dup_func;
+	GDestroyNotify g_destroy_func;
+	GeeCollection* _collection;
+};
+
 struct _GeeReadOnlyCollectionIterator {
 	GObject parent_instance;
 	GeeReadOnlyCollectionIteratorPrivate * priv;
@@ -43,15 +131,23 @@ struct _GeeReadOnlyCollectionIteratorClass {
 	GObjectClass parent_class;
 };
 
-
-
-struct _GeeReadOnlyCollectionPrivate {
-	GeeCollection* _collection;
+struct _GeeReadOnlyCollectionIteratorPrivate {
 	GType g_type;
 	GBoxedCopyFunc g_dup_func;
 	GDestroyNotify g_destroy_func;
 };
 
+
+static gpointer gee_read_only_collection_iterator_parent_class = NULL;
+static GeeIteratorIface* gee_read_only_collection_iterator_gee_iterator_parent_iface = NULL;
+static gpointer gee_read_only_collection_parent_class = NULL;
+static GeeIterableIface* gee_read_only_collection_gee_iterable_parent_iface = NULL;
+static GeeCollectionIface* gee_read_only_collection_gee_collection_parent_iface = NULL;
+
+GType gee_iterator_get_type (void);
+GType gee_iterable_get_type (void);
+GType gee_collection_get_type (void);
+GType gee_read_only_collection_get_type (void);
 #define GEE_READ_ONLY_COLLECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionPrivate))
 enum  {
 	GEE_READ_ONLY_COLLECTION_DUMMY_PROPERTY,
@@ -61,18 +157,22 @@ enum  {
 	GEE_READ_ONLY_COLLECTION_G_DUP_FUNC,
 	GEE_READ_ONLY_COLLECTION_G_DESTROY_FUNC
 };
+void gee_read_only_collection_set_collection (GeeReadOnlyCollection* self, GeeCollection* value);
+GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
+GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
+GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
 static GType gee_read_only_collection_real_get_element_type (GeeIterable* base);
+static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
+static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
+static GType gee_read_only_collection_iterator_get_type (void);
+GeeIterator* gee_iterable_iterator (GeeIterable* self);
 static GeeIterator* gee_read_only_collection_real_iterator (GeeIterable* base);
+gboolean gee_collection_contains (GeeCollection* self, gconstpointer item);
 static gboolean gee_read_only_collection_real_contains (GeeCollection* base, gconstpointer item);
 static gboolean gee_read_only_collection_real_add (GeeCollection* base, gconstpointer item);
 static gboolean gee_read_only_collection_real_remove (GeeCollection* base, gconstpointer item);
 static void gee_read_only_collection_real_clear (GeeCollection* base);
-struct _GeeReadOnlyCollectionIteratorPrivate {
-	GType g_type;
-	GBoxedCopyFunc g_dup_func;
-	GDestroyNotify g_destroy_func;
-};
-
+gint gee_collection_get_size (GeeCollection* self);
 #define GEE_READ_ONLY_COLLECTION_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorPrivate))
 enum  {
 	GEE_READ_ONLY_COLLECTION_ITERATOR_DUMMY_PROPERTY,
@@ -82,15 +182,12 @@ enum  {
 };
 static gboolean gee_read_only_collection_iterator_real_next (GeeIterator* base);
 static gpointer gee_read_only_collection_iterator_real_get (GeeIterator* base);
-static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
 static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
-static gpointer gee_read_only_collection_iterator_parent_class = NULL;
-static GeeIteratorIface* gee_read_only_collection_iterator_gee_iterator_parent_iface = NULL;
-static GType gee_read_only_collection_iterator_get_type (void);
-static gpointer gee_read_only_collection_parent_class = NULL;
-static GeeIterableIface* gee_read_only_collection_gee_iterable_parent_iface = NULL;
-static GeeCollectionIface* gee_read_only_collection_gee_collection_parent_iface = NULL;
+static void gee_read_only_collection_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
+static void gee_read_only_collection_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
 static void gee_read_only_collection_finalize (GObject* obj);
+static void gee_read_only_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
+static void gee_read_only_collection_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
 
 
 
@@ -130,33 +227,42 @@ GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFun
 
 static GType gee_read_only_collection_real_get_element_type (GeeIterable* base) {
 	GeeReadOnlyCollection * self;
+	GType result;
 	self = (GeeReadOnlyCollection*) base;
-	return self->priv->g_type;
+	result = self->priv->g_type;
+	return result;
 }
 
 
 static GeeIterator* gee_read_only_collection_real_iterator (GeeIterable* base) {
 	GeeReadOnlyCollection * self;
+	GeeIterator* result;
 	self = (GeeReadOnlyCollection*) base;
 	if (self->priv->_collection == NULL) {
-		return (GeeIterator*) gee_read_only_collection_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func);
+		result = (GeeIterator*) gee_read_only_collection_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func);
+		return result;
 	}
-	return gee_iterable_iterator ((GeeIterable*) self->priv->_collection);
+	result = gee_iterable_iterator ((GeeIterable*) self->priv->_collection);
+	return result;
 }
 
 
 static gboolean gee_read_only_collection_real_contains (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlyCollection * self;
+	gboolean result;
 	self = (GeeReadOnlyCollection*) base;
 	if (self->priv->_collection == NULL) {
-		return FALSE;
+		result = FALSE;
+		return result;
 	}
-	return gee_collection_contains (self->priv->_collection, item);
+	result = gee_collection_contains (self->priv->_collection, item);
+	return result;
 }
 
 
 static gboolean gee_read_only_collection_real_add (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlyCollection * self;
+	gboolean result;
 	self = (GeeReadOnlyCollection*) base;
 	g_assert_not_reached ();
 }
@@ -164,6 +270,7 @@ static gboolean gee_read_only_collection_real_add (GeeCollection* base, gconstpo
 
 static gboolean gee_read_only_collection_real_remove (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlyCollection * self;
+	gboolean result;
 	self = (GeeReadOnlyCollection*) base;
 	g_assert_not_reached ();
 }
@@ -177,34 +284,40 @@ static void gee_read_only_collection_real_clear (GeeCollection* base) {
 
 
 static gint gee_read_only_collection_real_get_size (GeeCollection* base) {
+	gint result;
 	GeeReadOnlyCollection* self;
 	self = (GeeReadOnlyCollection*) base;
-	return gee_collection_get_size (self->priv->_collection);
+	result = gee_collection_get_size (self->priv->_collection);
+	return result;
 }
 
 
 void gee_read_only_collection_set_collection (GeeReadOnlyCollection* self, GeeCollection* value) {
-	GeeCollection* _tmp1;
-	GeeCollection* _tmp0;
+	GeeCollection* _tmp1_;
+	GeeCollection* _tmp0_;
 	g_return_if_fail (self != NULL);
-	_tmp1 = NULL;
-	_tmp0 = NULL;
-	self->priv->_collection = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_collection == NULL) ? NULL : (self->priv->_collection = (g_object_unref (self->priv->_collection), NULL)), _tmp1);
+	_tmp1_ = NULL;
+	_tmp0_ = NULL;
+	self->priv->_collection = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_collection == NULL) ? NULL : (self->priv->_collection = (g_object_unref (self->priv->_collection), NULL)), _tmp1_);
 	g_object_notify ((GObject *) self, "collection");
 }
 
 
 static gboolean gee_read_only_collection_iterator_real_next (GeeIterator* base) {
 	GeeReadOnlyCollectionIterator * self;
+	gboolean result;
 	self = (GeeReadOnlyCollectionIterator*) base;
-	return FALSE;
+	result = FALSE;
+	return result;
 }
 
 
 static gpointer gee_read_only_collection_iterator_real_get (GeeIterator* base) {
 	GeeReadOnlyCollectionIterator * self;
+	gpointer result;
 	self = (GeeReadOnlyCollectionIterator*) base;
-	return NULL;
+	result = NULL;
+	return result;
 }
 
 
@@ -241,38 +354,6 @@ static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GTy
 }
 
 
-static void gee_read_only_collection_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
-	GeeReadOnlyCollectionIterator * self;
-	gpointer boxed;
-	self = GEE_READ_ONLY_COLLECTION_ITERATOR (object);
-	switch (property_id) {
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-	}
-}
-
-
-static void gee_read_only_collection_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
-	GeeReadOnlyCollectionIterator * self;
-	self = GEE_READ_ONLY_COLLECTION_ITERATOR (object);
-	switch (property_id) {
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-		case GEE_READ_ONLY_COLLECTION_ITERATOR_G_TYPE:
-		self->priv->g_type = g_value_get_gtype (value);
-		break;
-		case GEE_READ_ONLY_COLLECTION_ITERATOR_G_DUP_FUNC:
-		self->priv->g_dup_func = g_value_get_pointer (value);
-		break;
-		case GEE_READ_ONLY_COLLECTION_ITERATOR_G_DESTROY_FUNC:
-		self->priv->g_destroy_func = g_value_get_pointer (value);
-		break;
-	}
-}
-
-
 static void gee_read_only_collection_iterator_class_init (GeeReadOnlyCollectionIteratorClass * klass) {
 	gee_read_only_collection_iterator_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (GeeReadOnlyCollectionIteratorPrivate));
@@ -308,14 +389,11 @@ static GType gee_read_only_collection_iterator_get_type (void) {
 }
 
 
-static void gee_read_only_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
-	GeeReadOnlyCollection * self;
+static void gee_read_only_collection_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
+	GeeReadOnlyCollectionIterator * self;
 	gpointer boxed;
-	self = GEE_READ_ONLY_COLLECTION (object);
+	self = GEE_READ_ONLY_COLLECTION_ITERATOR (object);
 	switch (property_id) {
-		case GEE_READ_ONLY_COLLECTION_SIZE:
-		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
-		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -323,23 +401,20 @@ static void gee_read_only_collection_get_property (GObject * object, guint prope
 }
 
 
-static void gee_read_only_collection_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
-	GeeReadOnlyCollection * self;
-	self = GEE_READ_ONLY_COLLECTION (object);
+static void gee_read_only_collection_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
+	GeeReadOnlyCollectionIterator * self;
+	self = GEE_READ_ONLY_COLLECTION_ITERATOR (object);
 	switch (property_id) {
-		case GEE_READ_ONLY_COLLECTION_COLLECTION:
-		gee_read_only_collection_set_collection (self, g_value_get_object (value));
-		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
-		case GEE_READ_ONLY_COLLECTION_G_TYPE:
+		case GEE_READ_ONLY_COLLECTION_ITERATOR_G_TYPE:
 		self->priv->g_type = g_value_get_gtype (value);
 		break;
-		case GEE_READ_ONLY_COLLECTION_G_DUP_FUNC:
+		case GEE_READ_ONLY_COLLECTION_ITERATOR_G_DUP_FUNC:
 		self->priv->g_dup_func = g_value_get_pointer (value);
 		break;
-		case GEE_READ_ONLY_COLLECTION_G_DESTROY_FUNC:
+		case GEE_READ_ONLY_COLLECTION_ITERATOR_G_DESTROY_FUNC:
 		self->priv->g_destroy_func = g_value_get_pointer (value);
 		break;
 	}
@@ -404,5 +479,43 @@ GType gee_read_only_collection_get_type (void) {
 }
 
 
+static void gee_read_only_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
+	GeeReadOnlyCollection * self;
+	gpointer boxed;
+	self = GEE_READ_ONLY_COLLECTION (object);
+	switch (property_id) {
+		case GEE_READ_ONLY_COLLECTION_SIZE:
+		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
+		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+
+static void gee_read_only_collection_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
+	GeeReadOnlyCollection * self;
+	self = GEE_READ_ONLY_COLLECTION (object);
+	switch (property_id) {
+		case GEE_READ_ONLY_COLLECTION_COLLECTION:
+		gee_read_only_collection_set_collection (self, g_value_get_object (value));
+		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+		case GEE_READ_ONLY_COLLECTION_G_TYPE:
+		self->priv->g_type = g_value_get_gtype (value);
+		break;
+		case GEE_READ_ONLY_COLLECTION_G_DUP_FUNC:
+		self->priv->g_dup_func = g_value_get_pointer (value);
+		break;
+		case GEE_READ_ONLY_COLLECTION_G_DESTROY_FUNC:
+		self->priv->g_destroy_func = g_value_get_pointer (value);
+		break;
+	}
+}
+
+
 
 
