@@ -20,9 +20,53 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gee/readonlyset.h>
+#include <glib.h>
+#include <glib-object.h>
 
 
+#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
+#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
+#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
+#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))
+
+typedef struct _GeeIterable GeeIterable;
+typedef struct _GeeIterableIface GeeIterableIface;
+
+#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
+#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
+#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
+#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))
+
+typedef struct _GeeIterator GeeIterator;
+typedef struct _GeeIteratorIface GeeIteratorIface;
+
+#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
+#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
+#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
+#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))
+
+typedef struct _GeeCollection GeeCollection;
+typedef struct _GeeCollectionIface GeeCollectionIface;
+
+#define GEE_TYPE_SET (gee_set_get_type ())
+#define GEE_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SET, GeeSet))
+#define GEE_IS_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SET))
+#define GEE_SET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_SET, GeeSetIface))
+
+typedef struct _GeeSet GeeSet;
+typedef struct _GeeSetIface GeeSetIface;
+
+#define GEE_TYPE_READ_ONLY_SET (gee_read_only_set_get_type ())
+#define GEE_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySet))
+#define GEE_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))
+#define GEE_IS_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SET))
+#define GEE_IS_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SET))
+#define GEE_READ_ONLY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))
+
+typedef struct _GeeReadOnlySet GeeReadOnlySet;
+typedef struct _GeeReadOnlySetClass GeeReadOnlySetClass;
+typedef struct _GeeReadOnlySetPrivate GeeReadOnlySetPrivate;
+
 #define GEE_READ_ONLY_SET_TYPE_ITERATOR (gee_read_only_set_iterator_get_type ())
 #define GEE_READ_ONLY_SET_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIterator))
 #define GEE_READ_ONLY_SET_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIteratorClass))
@@ -34,6 +78,65 @@ typedef struct _GeeReadOnlySetIterator GeeReadOnlySetIterator;
 typedef struct _GeeReadOnlySetIteratorClass GeeReadOnlySetIteratorClass;
 typedef struct _GeeReadOnlySetIteratorPrivate GeeReadOnlySetIteratorPrivate;
 
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIteratorIface {
+	GTypeInterface parent_iface;
+	gboolean (*next) (GeeIterator* self);
+	gpointer (*get) (GeeIterator* self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIterableIface {
+	GTypeInterface parent_iface;
+	GType (*get_element_type) (GeeIterable* self);
+	GeeIterator* (*iterator) (GeeIterable* self);
+};
+
+/**
+ * Serves as the base interface for implementing collection classes. Defines
+ * size, iteration, and modification methods.
+ */
+struct _GeeCollectionIface {
+	GTypeInterface parent_iface;
+	gboolean (*contains) (GeeCollection* self, gconstpointer item);
+	gboolean (*add) (GeeCollection* self, gconstpointer item);
+	gboolean (*remove) (GeeCollection* self, gconstpointer item);
+	void (*clear) (GeeCollection* self);
+	gint (*get_size) (GeeCollection* self);
+};
+
+/**
+ * A set is a collection without duplicates.
+ */
+struct _GeeSetIface {
+	GTypeInterface parent_iface;
+};
+
+/**
+ * Represents a read-only collection of items without duplicates.
+ */
+struct _GeeReadOnlySet {
+	GObject parent_instance;
+	GeeReadOnlySetPrivate * priv;
+};
+
+struct _GeeReadOnlySetClass {
+	GObjectClass parent_class;
+};
+
+struct _GeeReadOnlySetPrivate {
+	GType g_type;
+	GBoxedCopyFunc g_dup_func;
+	GDestroyNotify g_destroy_func;
+	GeeSet* _set;
+};
+
 struct _GeeReadOnlySetIterator {
 	GObject parent_instance;
 	GeeReadOnlySetIteratorPrivate * priv;
@@ -43,15 +146,25 @@ struct _GeeReadOnlySetIteratorClass {
 	GObjectClass parent_class;
 };
 
-
-
-struct _GeeReadOnlySetPrivate {
-	GeeSet* _set;
+struct _GeeReadOnlySetIteratorPrivate {
 	GType g_type;
 	GBoxedCopyFunc g_dup_func;
 	GDestroyNotify g_destroy_func;
 };
 
+
+static gpointer gee_read_only_set_iterator_parent_class = NULL;
+static GeeIteratorIface* gee_read_only_set_iterator_gee_iterator_parent_iface = NULL;
+static gpointer gee_read_only_set_parent_class = NULL;
+static GeeIterableIface* gee_read_only_set_gee_iterable_parent_iface = NULL;
+static GeeCollectionIface* gee_read_only_set_gee_collection_parent_iface = NULL;
+static GeeSetIface* gee_read_only_set_gee_set_parent_iface = NULL;
+
+GType gee_iterator_get_type (void);
+GType gee_iterable_get_type (void);
+GType gee_collection_get_type (void);
+GType gee_set_get_type (void);
+GType gee_read_only_set_get_type (void);
 #define GEE_READ_ONLY_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetPrivate))
 enum  {
 	GEE_READ_ONLY_SET_DUMMY_PROPERTY,
@@ -61,18 +174,22 @@ enum  {
 	GEE_READ_ONLY_SET_G_DUP_FUNC,
 	GEE_READ_ONLY_SET_G_DESTROY_FUNC
 };
+void gee_read_only_set_set_set (GeeReadOnlySet* self, GeeSet* value);
+GeeReadOnlySet* gee_read_only_set_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set);
+GeeReadOnlySet* gee_read_only_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set);
+GeeReadOnlySet* gee_read_only_set_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set);
 static GType gee_read_only_set_real_get_element_type (GeeIterable* base);
+static GeeReadOnlySetIterator* gee_read_only_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
+static GeeReadOnlySetIterator* gee_read_only_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
+static GType gee_read_only_set_iterator_get_type (void);
+GeeIterator* gee_iterable_iterator (GeeIterable* self);
 static GeeIterator* gee_read_only_set_real_iterator (GeeIterable* base);
+gboolean gee_collection_contains (GeeCollection* self, gconstpointer item);
 static gboolean gee_read_only_set_real_contains (GeeCollection* base, gconstpointer item);
 static gboolean gee_read_only_set_real_add (GeeCollection* base, gconstpointer item);
 static gboolean gee_read_only_set_real_remove (GeeCollection* base, gconstpointer item);
 static void gee_read_only_set_real_clear (GeeCollection* base);
-struct _GeeReadOnlySetIteratorPrivate {
-	GType g_type;
-	GBoxedCopyFunc g_dup_func;
-	GDestroyNotify g_destroy_func;
-};
-
+gint gee_collection_get_size (GeeCollection* self);
 #define GEE_READ_ONLY_SET_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIteratorPrivate))
 enum  {
 	GEE_READ_ONLY_SET_ITERATOR_DUMMY_PROPERTY,
@@ -82,16 +199,12 @@ enum  {
 };
 static gboolean gee_read_only_set_iterator_real_next (GeeIterator* base);
 static gpointer gee_read_only_set_iterator_real_get (GeeIterator* base);
-static GeeReadOnlySetIterator* gee_read_only_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
 static GeeReadOnlySetIterator* gee_read_only_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
-static gpointer gee_read_only_set_iterator_parent_class = NULL;
-static GeeIteratorIface* gee_read_only_set_iterator_gee_iterator_parent_iface = NULL;
-static GType gee_read_only_set_iterator_get_type (void);
-static gpointer gee_read_only_set_parent_class = NULL;
-static GeeIterableIface* gee_read_only_set_gee_iterable_parent_iface = NULL;
-static GeeCollectionIface* gee_read_only_set_gee_collection_parent_iface = NULL;
-static GeeSetIface* gee_read_only_set_gee_set_parent_iface = NULL;
+static void gee_read_only_set_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
+static void gee_read_only_set_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
 static void gee_read_only_set_finalize (GObject* obj);
+static void gee_read_only_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
+static void gee_read_only_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
 
 
 
@@ -131,33 +244,42 @@ GeeReadOnlySet* gee_read_only_set_new (GType g_type, GBoxedCopyFunc g_dup_func,
 
 static GType gee_read_only_set_real_get_element_type (GeeIterable* base) {
 	GeeReadOnlySet * self;
+	GType result;
 	self = (GeeReadOnlySet*) base;
-	return self->priv->g_type;
+	result = self->priv->g_type;
+	return result;
 }
 
 
 static GeeIterator* gee_read_only_set_real_iterator (GeeIterable* base) {
 	GeeReadOnlySet * self;
+	GeeIterator* result;
 	self = (GeeReadOnlySet*) base;
 	if (self->priv->_set == NULL) {
-		return (GeeIterator*) gee_read_only_set_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func);
+		result = (GeeIterator*) gee_read_only_set_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func);
+		return result;
 	}
-	return gee_iterable_iterator ((GeeIterable*) self->priv->_set);
+	result = gee_iterable_iterator ((GeeIterable*) self->priv->_set);
+	return result;
 }
 
 
 static gboolean gee_read_only_set_real_contains (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlySet * self;
+	gboolean result;
 	self = (GeeReadOnlySet*) base;
 	if (self->priv->_set == NULL) {
-		return FALSE;
+		result = FALSE;
+		return result;
 	}
-	return gee_collection_contains ((GeeCollection*) self->priv->_set, item);
+	result = gee_collection_contains ((GeeCollection*) self->priv->_set, item);
+	return result;
 }
 
 
 static gboolean gee_read_only_set_real_add (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlySet * self;
+	gboolean result;
 	self = (GeeReadOnlySet*) base;
 	g_assert_not_reached ();
 }
@@ -165,6 +287,7 @@ static gboolean gee_read_only_set_real_add (GeeCollection* base, gconstpointer i
 
 static gboolean gee_read_only_set_real_remove (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlySet * self;
+	gboolean result;
 	self = (GeeReadOnlySet*) base;
 	g_assert_not_reached ();
 }
@@ -178,34 +301,40 @@ static void gee_read_only_set_real_clear (GeeCollection* base) {
 
 
 static gint gee_read_only_set_real_get_size (GeeCollection* base) {
+	gint result;
 	GeeReadOnlySet* self;
 	self = (GeeReadOnlySet*) base;
-	return gee_collection_get_size ((GeeCollection*) self->priv->_set);
+	result = gee_collection_get_size ((GeeCollection*) self->priv->_set);
+	return result;
 }
 
 
 void gee_read_only_set_set_set (GeeReadOnlySet* self, GeeSet* value) {
-	GeeSet* _tmp1;
-	GeeSet* _tmp0;
+	GeeSet* _tmp1_;
+	GeeSet* _tmp0_;
 	g_return_if_fail (self != NULL);
-	_tmp1 = NULL;
-	_tmp0 = NULL;
-	self->priv->_set = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_set == NULL) ? NULL : (self->priv->_set = (g_object_unref (self->priv->_set), NULL)), _tmp1);
+	_tmp1_ = NULL;
+	_tmp0_ = NULL;
+	self->priv->_set = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_set == NULL) ? NULL : (self->priv->_set = (g_object_unref (self->priv->_set), NULL)), _tmp1_);
 	g_object_notify ((GObject *) self, "set");
 }
 
 
 static gboolean gee_read_only_set_iterator_real_next (GeeIterator* base) {
 	GeeReadOnlySetIterator * self;
+	gboolean result;
 	self = (GeeReadOnlySetIterator*) base;
-	return FALSE;
+	result = FALSE;
+	return result;
 }
 
 
 static gpointer gee_read_only_set_iterator_real_get (GeeIterator* base) {
 	GeeReadOnlySetIterator * self;
+	gpointer result;
 	self = (GeeReadOnlySetIterator*) base;
-	return NULL;
+	result = NULL;
+	return result;
 }
 
 
@@ -242,38 +371,6 @@ static GeeReadOnlySetIterator* gee_read_only_set_iterator_new (GType g_type, GBo
 }
 
 
-static void gee_read_only_set_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
-	GeeReadOnlySetIterator * self;
-	gpointer boxed;
-	self = GEE_READ_ONLY_SET_ITERATOR (object);
-	switch (property_id) {
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-	}
-}
-
-
-static void gee_read_only_set_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
-	GeeReadOnlySetIterator * self;
-	self = GEE_READ_ONLY_SET_ITERATOR (object);
-	switch (property_id) {
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-		case GEE_READ_ONLY_SET_ITERATOR_G_TYPE:
-		self->priv->g_type = g_value_get_gtype (value);
-		break;
-		case GEE_READ_ONLY_SET_ITERATOR_G_DUP_FUNC:
-		self->priv->g_dup_func = g_value_get_pointer (value);
-		break;
-		case GEE_READ_ONLY_SET_ITERATOR_G_DESTROY_FUNC:
-		self->priv->g_destroy_func = g_value_get_pointer (value);
-		break;
-	}
-}
-
-
 static void gee_read_only_set_iterator_class_init (GeeReadOnlySetIteratorClass * klass) {
 	gee_read_only_set_iterator_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (GeeReadOnlySetIteratorPrivate));
@@ -309,14 +406,11 @@ static GType gee_read_only_set_iterator_get_type (void) {
 }
 
 
-static void gee_read_only_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
-	GeeReadOnlySet * self;
+static void gee_read_only_set_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
+	GeeReadOnlySetIterator * self;
 	gpointer boxed;
-	self = GEE_READ_ONLY_SET (object);
+	self = GEE_READ_ONLY_SET_ITERATOR (object);
 	switch (property_id) {
-		case GEE_READ_ONLY_SET_SIZE:
-		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
-		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -324,23 +418,20 @@ static void gee_read_only_set_get_property (GObject * object, guint property_id,
 }
 
 
-static void gee_read_only_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
-	GeeReadOnlySet * self;
-	self = GEE_READ_ONLY_SET (object);
+static void gee_read_only_set_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
+	GeeReadOnlySetIterator * self;
+	self = GEE_READ_ONLY_SET_ITERATOR (object);
 	switch (property_id) {
-		case GEE_READ_ONLY_SET_SET:
-		gee_read_only_set_set_set (self, g_value_get_object (value));
-		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
-		case GEE_READ_ONLY_SET_G_TYPE:
+		case GEE_READ_ONLY_SET_ITERATOR_G_TYPE:
 		self->priv->g_type = g_value_get_gtype (value);
 		break;
-		case GEE_READ_ONLY_SET_G_DUP_FUNC:
+		case GEE_READ_ONLY_SET_ITERATOR_G_DUP_FUNC:
 		self->priv->g_dup_func = g_value_get_pointer (value);
 		break;
-		case GEE_READ_ONLY_SET_G_DESTROY_FUNC:
+		case GEE_READ_ONLY_SET_ITERATOR_G_DESTROY_FUNC:
 		self->priv->g_destroy_func = g_value_get_pointer (value);
 		break;
 	}
@@ -412,5 +503,43 @@ GType gee_read_only_set_get_type (void) {
 }
 
 
+static void gee_read_only_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
+	GeeReadOnlySet * self;
+	gpointer boxed;
+	self = GEE_READ_ONLY_SET (object);
+	switch (property_id) {
+		case GEE_READ_ONLY_SET_SIZE:
+		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
+		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+
+static void gee_read_only_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
+	GeeReadOnlySet * self;
+	self = GEE_READ_ONLY_SET (object);
+	switch (property_id) {
+		case GEE_READ_ONLY_SET_SET:
+		gee_read_only_set_set_set (self, g_value_get_object (value));
+		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+		case GEE_READ_ONLY_SET_G_TYPE:
+		self->priv->g_type = g_value_get_gtype (value);
+		break;
+		case GEE_READ_ONLY_SET_G_DUP_FUNC:
+		self->priv->g_dup_func = g_value_get_pointer (value);
+		break;
+		case GEE_READ_ONLY_SET_G_DESTROY_FUNC:
+		self->priv->g_destroy_func = g_value_get_pointer (value);
+		break;
+	}
+}
+
+
 
 
