@@ -412,14 +412,14 @@ static void _test_hashmap_clear_gcallback (void) {
 
 void _main (char** args, int args_length1) {
 	g_test_init (&args_length1, &args, NULL);
-	g_test_add_func ("/HashMap/Map/get", _test_hashmap_get_gcallback);
-	g_test_add_func ("/HashMap/Map/set", _test_hashmap_set_gcallback);
-	g_test_add_func ("/HashMap/Map/remove", _test_hashmap_remove_gcallback);
-	g_test_add_func ("/HashMap/Map/contains", _test_hashmap_contains_gcallback);
-	g_test_add_func ("/HashMap/Map/size", _test_hashmap_size_gcallback);
-	g_test_add_func ("/HashMap/Map/get_keys", _test_hashmap_get_keys_gcallback);
-	g_test_add_func ("/HashMap/Map/get_values", _test_hashmap_get_values_gcallback);
-	g_test_add_func ("/HashMap/Map/clear", _test_hashmap_clear_gcallback);
+	g_test_add_func ("/HashMap/Map/get", (GCallback) _test_hashmap_get_gcallback);
+	g_test_add_func ("/HashMap/Map/set", (GCallback) _test_hashmap_set_gcallback);
+	g_test_add_func ("/HashMap/Map/remove", (GCallback) _test_hashmap_remove_gcallback);
+	g_test_add_func ("/HashMap/Map/contains", (GCallback) _test_hashmap_contains_gcallback);
+	g_test_add_func ("/HashMap/Map/size", (GCallback) _test_hashmap_size_gcallback);
+	g_test_add_func ("/HashMap/Map/get_keys", (GCallback) _test_hashmap_get_keys_gcallback);
+	g_test_add_func ("/HashMap/Map/get_values", (GCallback) _test_hashmap_get_values_gcallback);
+	g_test_add_func ("/HashMap/Map/clear", (GCallback) _test_hashmap_clear_gcallback);
 	g_test_run ();
 }
 
